#!/usr/bin/perl -w

# Written by Simon Huggins
# (C) Project Purple 1999
# 21/7/1999 - started
# 24/7/1999 - sorted out my sendmail config so I didn't have to be online to
# 	      test it and thus made some progress :-)
 
use strict;
use English;
use Net::POP3;
use Net::SMTP;
use Carp;

print "Filtermail v0.0.1 (C) Project Purple 1999 released under the GPL\n";

my ($pop, $msgsref, $nummsgs, $verbosity, $headerref, $do);
my ($smtp, $bodyref, $to, $from, %config, @msgidcache, $idline);

# One action per regexp (when regexp matches return action)
my (@actions, @regexps, $num_actions, @removeactions);

$num_actions=0;

# Action	Meaning
# 
# 0		Leave on server, add to cache, get users idea.
# 1		Get.
# 2		Delete.
# 3		Leave, but already in cache.

sub getactions_from_file() {
	my $name=shift;

	if (open(INST, "<$name")) {
	while (<INST>) {
		if (/([0-9])\s+(.*)/) {
			push @actions, $1;
			push @regexps, $2;
			$num_actions++;
		}
	}
	close(INST);
	}
}

sub filter() {
	my $msgsize=shift;
	my @headers=@_;
	my ($line,$i);

	# Check headers against hash of headers to check.

	for $line (@headers) {
		for ($i=0; $i<$num_actions; $i++) {
			if ($line =~ /$regexps[$i]/) {
				push @removeactions, $regexps[$i];
				return $actions[$i];
			}
		}
	}
	return 0;	# Default is to cache and await instructions
}

### Do our own ARGV parsing cos diving into getopts looks like a pita
### just for one arg.

$verbosity=0;
for (@ARGV) {
	if (/-(v+)/) {
		$verbosity += length($1);
	} else {
		print "Invalid option $_ given on commandline!\n";
		croak "Usage: $0 [-v]\n";
	}
}

if ($verbosity>0) { print "Being verbose (" . $verbosity . ")\n"; }

### Should read a config file for now just stuff the hash

$config{'to'}="huggie\@localhost";

### Read the instructions for Message IDs

&getactions_from_file('pre_instructions'); 	# FIXME needs to be better
&getactions_from_file('instructions'); 		# algorithm really 
&getactions_from_file('post_instructions');	# should be clever/more
						# flexible.

### Read the cache and make up a cache of msgids - don't add anything to the
### cache twice

if (open(CACHE, '<cache')) {
	while (<CACHE>) {
		if (/Message-ID:/i) { push @msgidcache, $_; }
	}
	close(CACHE);
}

### Establish a POP3 connection

$pop=Net::POP3->new('localhost', Timeout => 60);

$nummsgs=$pop->login("poptest","Pahyouthoughtidtellyouapasswordforauseronmybox?!?!");
if (not defined $nummsgs) { croak "Couldn't authenticate poptest\n";}
if ($nummsgs == 0) { croak "No mail for user poptest!\n"; }
if ($verbosity>1) { print "$nummsgs messages on server\n"; }


### Get the LIST of message number and length (octets)

$msgsref=$pop->list();
$|=1;
MSG: for (keys %$msgsref) {
	if ($verbosity>2) {
		#	Number	Size (in octets)
		print "$_\t" . $$msgsref{$_} . "\n";
	}
	$headerref=$pop->top($_, 0); 	# Send a TOP and return an arrayref to
					# the headers
	if ($verbosity>3) {
		print "Headers of message $_\n@$headerref";
	}
	
	# Call filter on the array of headers

	$do=&filter($_,@$headerref);

	# do the right thing by $do

	for (@$headerref) {
		if (/^To: (.*)/i) { $to=$1; }
		if (/^From: (.*)/i) { $from=$1; }
		if (/^Message-ID:/i) { $idline=$_; }
	}

	if ($do==0) {
		my $num=$_;

		for (@msgidcache) {
			if (/^$idline$/) {
				print "Had already seen $num\n";
				next MSG;
			}
		}

		if ($verbosity>2) {
			print "Left $num and cached its ID\n";
		}
		# Leave and cache it
		open(CACHE, ">>cache");
		for (@$headerref) {
			if (/^(In-Reply-)?To:/i) { print CACHE $_; }
			if (/^From:/i) { print CACHE $_; }
			if (/^Message-ID:/i) { print CACHE $_; }
			if (/^References:/i) { print CACHE $_; }
			if (/^Organi[sz]ation:/i) { print CACHE $_; }
			if (/^Subject:/i) { print CACHE $_; }
			if (/^Sender:/i) { print CACHE $_; }
			if (/^X-Mailer:/i) { print CACHE $_; }
			if (/^Cc:/i) { print CACHE $_; }
		}
		print CACHE "---SIZE=$$msgsref{$num}\n";
		close CACHE;
		next;
	} elsif ($do==1) {
		if ($verbosity>2) {
			print "Retrieved $_\n";
		}
		# Get it
		$bodyref=$pop->get($_);
		if (not defined $smtp) { 			# then open a connection
			$smtp=Net::SMTP->new('localhost', Timeout => 60,
							  Hello => 'localhost');
			if ($verbosity>2) {
				print "Opened SMTP connection\n";
			}
			}

		$smtp->mail($from);
		$smtp->to($config{'to'});

		$smtp->data();
		$smtp->datasend(@$bodyref);
		$smtp->dataend();
		if ($verbosity>2) {
			print "Sent message $_\n";
		}
		#$pop->delete($_);
	} elsif ($do==2) {
		if ($verbosity>2) {
			print "Deleted $_\n";
		}
		# Delete it
		$pop->delete($_);
	} elsif ($do==3) {
		if ($verbosity>2) {
			print "Left $_ on server.\n";
		}
		# Leave it
		next;
	}
}

$pop->quit();
if (defined $smtp) { $smtp->quit(); }

if ($verbosity>2) { print "Hung up from pop3 server!\nCleaning up.\n"; }

### Look through instructions (but *NOT* pre and post) for items in @remove
### NB pre and post are meant for persistant rules, e.g. mailing lists
### instructions just for per message ones.

if (scalar @removeactions>0) {
	my (@ins,$i,$r,$changed,@newins);
	$changed=0;
	if (open(INST, '<instructions')) {
		@ins=<INST>;
		close(INST);
		@newins=();
		for $i (@ins) {
			chomp($i);
			for $r (@removeactions) {
print "i = #$i#\nr = #$r#";
				$r=quotemeta($r); # ick, but $r contains the
						  # regexp stuff like ^ so it
						  # needs quoted
				if (not $i =~ /$r/) {
					push @newins, $i;
				} else { $changed=1; }
			}
		}
print "$changed\n";
		if ($changed) {
print "Changed!\n";
			open(INST, '>instructions');
			print INST @newins;
			close(INST);
		}
	}
}

print "Done.\n";
